;	This program is free software; you can redistribute it and/or modify
;	it under the terms of the GNU General Public License as published by
;	the Free Software Foundation; either version 2 of the License, or
;	(at your option) any later version.
;
;	This program is distributed in the hope that it will be useful,
;	but WITHOUT ANY WARRANTY; without even the implied warranty of
;	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;	GNU General Public License for more details.
;
;	You should have received a copy of the GNU General Public License
;	along with this program; if not, write to the Free Software
;	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
BCD_Add:
	;Inputs - HL and DE point to two numbers, in my packed BCD format.
	;Outputs - the result of the addition is created as temporary variable, pointer is in HL
	;
	;First we must do some checking of the signs
	inc hl
	ld a,(hl)
	dec hl
	and 80h
	jr nz,BCD_Add_HLNegative
	;HL is positive...check DE
	inc de
	ld a,(de)
	dec de
	and 80h
	jr z,BCD_Add_Start		;HL and DE are both positive
	;DE is negative...
	jp BCD_Subtract_Start
BCD_Add_HLNegative:
	;HL is negative...check DE
	inc de
	ld a,(de)
	dec de
	and 80h
	jr z,BCD_Add_DEPositive
	;DE and HL both negative
	call BCD_Add_Start
	inc hl
	set 7,(hl)
	dec hl
	ret
BCD_Add_DEPositive:
	ex de,hl
	jp BCD_Subtract_Start
BCD_Add_Start:
	push hl
	ld c,(hl)
	inc hl
	ld b,(hl)
	res 7,b
	;BC is the size of HL
	ld a,(de)
	ld l,a
	inc de
	ld a,(de)
	res 7,a
	ld h,a
	dec de
	;DE is unchanged now, HL is the size of the (DE) string, BC is the size of the original
	;(HL) string
	ld (temp1),bc
	ld (temp2),hl
	or a
	sbc hl,bc
	jr c,BCD_Add_Swap
	pop ix
	push de
	push ix
	pop de
	add hl,bc
	ld a,h
	ld h,b
	ld b,a
	ld a,l
	ld l,c
	ld c,a ;switch HL and BC
	or a
	sbc hl,bc
BCD_Add_Swap:
	add hl,bc
	ld a,h
	ld h,b
	ld b,a
	ld a,l
	ld l,c
	ld c,a ;switch HL and BC
	;BC is now the smaller string length, HL is the larger
	ld (temp1),bc
	ld (temp2),hl
	push de
	ld de,(areaFreeStart)
	push de
	pop ix
	add hl,de
	inc hl
	inc hl
	inc hl
	or a
	ld de,(areaFreeEnd)
	inc de
	sbc hl,de
	jp nc,MemoryError
	add hl,de
	ld (areaFreeStart),hl
	pop de
	;DE original string, IX is the location of the result
	ld hl,(temp2)
	inc hl
	ld (ix),l
	ld (ix+1),h
	pop hl
	inc ix
	inc ix
	ld (ix),0
	inc ix
	;HL, DE original strings, all ready to go...rightmost byte
	ld c,(hl)
	inc hl
	ld b,(hl)
	res 7,b
	inc hl
	push de
	push ix
	pop de
	ld (temp2),de			;save the new string pointer
	push bc
	ldir		;copy the longer string to the result...then we'll add the shorter string on
	pop bc
	dec de
	ex de,hl
	pop de
	;push bc
	push hl
	ld a,(de)
	ld c,a
	inc de
	ld a,(de)
	res 7,a
	ld b,a
	ex de,hl
	add hl,bc
	ex de,hl
	ld bc,(temp1)	;BC is how many bytes to add
	pop hl
	;pop bc
	;HL is the rightmost byte of the first number....DE is the rightmost byte of the second
	;BC is the number of bytes to add
	or a	;clear carry
	push af
BCD_Add_Loop:
	pop af
	ld a,(de)
	adc a,(hl)
	daa
	push af
	ld (hl),a
	dec hl
	dec de
	dec bc
	ld a,b
	or c
	jr nz,BCD_Add_Loop
	pop af
BCD_Add_Loop2:
	ld a,0
	adc a,(hl)
	daa
	ld (hl),a
	dec hl
	jr c,BCD_Add_Loop2
	;ld a,0
	;adc a,(hl)
	;daa
	;ld (hl),a
	;One more thing...if that first byte we added is still completely 0, let's remove it.
	ld hl,(temp2)
	dec hl
	ld a,(hl)
	or a
	jr nz,BCD_Add_NoShiftNeeded
	;The first byte is 0
	dec hl
	ld b,(hl)
	res 7,b
	dec hl
	push hl
	ld c,(hl)
	dec bc
	ld (hl),c
	inc hl
	ld (hl),b
	;BC is the new length
	inc hl
	ld d,h
	ld e,l
	inc hl
	ldir			;shift it down
	;Now fix the free RAM area...
	ld hl,(areaFreeEnd)
	dec hl
	ld (areaFreeEnd),hl
	pop hl			;HL points to the result...first byte of it
	ret		;all done!
BCD_Add_NoShiftNeeded:
	dec hl
	dec hl
	ret