;	This program is free software; you can redistribute it and/or modify
;	it under the terms of the GNU General Public License as published by
;	the Free Software Foundation; either version 2 of the License, or
;	(at your option) any later version.
;
;	This program is distributed in the hope that it will be useful,
;	but WITHOUT ANY WARRANTY; without even the implied warranty of
;	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;	GNU General Public License for more details.
;
;	You should have received a copy of the GNU General Public License
;	along with this program; if not, write to the Free Software
;	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

; Display routines
BCD_Display:
	;Input - BCD number pointer in HL
	;Output - The number is displayed in the large font...until Enter is pressed if it's long enough
	ld (displayPointer),hl
	ld bc,0
	ld (displayOffset),bc
	ld (displayStart),bc
	ld c,(hl)
	inc hl
	ld b,(hl)
	ld a,b
	res 7,b
	and 10000000b
	ld (displayFlags),a
	push hl
	add hl,bc
	ld (displayLastByte),hl
	pop hl
	inc hl
	ld a,(hl)
	and 11110000b
	ld hl,displayFlags
	jr nz,BCD_Display_No6
	set 6,(hl)
BCD_Display_No6:
	;displayFlags - bit 7 is the sign...bit 6 is set if the first nibble is zero
	bit 7,(hl)
	jp z,BCD_Display_Section_Initial
	ld bc,-1
	ld (displayOffset),bc
	ld (displayStart),bc
	jp BCD_Display_Section
;BCD_Display_Loop:
;	in a,(4)
;	bit 3,a
;	jr z,BCD_Display_Turbo
;	B_CALL GetKey
;	cp kLeft
;	jr z,BCD_Display_Left
;	cp kRight
;	jr z,BCD_Display_Right
;	cp kEnter
;	jr nz,BCD_Display_Loop
;BCD_Display_Exit:
;	ret
;BCD_Display_Turbo:
;	in a,(4)
;	bit 3,a
;	jr nz,BCD_Display_Loop
;	;Fast scroll
;	ld a,0FFh
;	out (1),a
;	nop
;	nop
;	ld a,0FEh
;	out (1),a
;	nop
;	nop
;	in a,(1)
;	cp 0FBh
;	jr z,BCD_Display_Right
;	cp 0FDh
;	jr nz,BCD_Display_Loop
BCD_Display_Left:
	ld de,(displayOffset)
	ld hl,(displayStart)
	or a
	sbc hl,de
	ret z
	ld hl,0001h
	or a
	sbc hl,de
	dec de
	jr nz,BCD_Display_Left_End2
	ld hl,displayFlags
	bit 6,(hl)
	jr z,BCD_Display_Left_End2
	ld de,(displayStart)
BCD_Display_Left_End2:
	ld (displayOffset),de
BCD_Display_Left_End:
	jp BCD_Display_Section
BCD_Display_Right:
	ld de,(displayOffset)
	inc de
	srl d
	rr e
	ld bc,7
	ld hl,(displayPointer)
	add hl,bc
	add hl,de
	ex de,hl
	ld hl,(displayLastByte)
	or a
	sbc hl,de
	ret c
	ld de,(displayOffset)
	call BCD_Display_ScrollDERight
	ld (displayOffset),de
	jr BCD_Display_Left_End
BCD_Display_ScrollDERight:
	inc de
	ld a,d
	or e
	ret nz
	ld hl,displayFlags
	bit 6,(hl)
	ret z
	inc de
	ret
BCD_Display_Section_Initial:
	ld hl,displayFlags
	bit 6,(hl)
	jr z,BCD_Display_Section
	ld de,1
	ld (displayOffset),de
	ld (displayStart),de
BCD_Display_Section:
	;we assume (currow) is set...
	;Z if all of the number fit on the screen
	ld a,2
	ld (curcol),a
	ld de,(displayOffset)
	ld hl,(displayStart)
	or a
	sbc hl,de
	jr z,BCD_Display_Section_NoLeftArrow
	ld a,0CFh
	B_CALL PutC
	jr BCD_Display_Section_Go
BCD_Display_Section_NoLeftArrow:
	;If it is the start of the number
	ld a,' '
	B_CALL PutC
BCD_Display_Section_Go:
	ld de,(displayOffset)
	ld hl,0FFFFh
	or a
	sbc hl,de
	jr nz,BCD_Display_Section_AfterFirst
	push de
	ld a,lneg
	B_CALL PutC
	pop de
	call BCD_Display_ScrollDERight
BCD_Display_Section_AfterFirst:
	;Okay now we are ready to show as many digits as possible, until
	;we reach (curcol)=14 or we run out of bytes
	;DE is the starting offset in digits
	ld (temp8),de
BCD_Display_Section_Loop:
	ld de,(temp8)
	srl d
	rr e
	ld hl,(displayPointer)
	inc hl
	inc hl
	add hl,de
	;HL now is the byte where we want it..
	ld a,(temp8)
	and 1
	jr z,BCD_Display_UpperNibble
	ld a,(hl)
	and 00001111b
	jr BCD_Display_LoopAround
BCD_Display_UpperNibble:
	ld a,(hl)
	srl a
	srl a
	srl a
	srl a
BCD_Display_LoopAround:
	add a,'0'
	B_CALL PutC
	ld de,(temp8)
	inc de
	ld (temp8),de
	sra d
	rr e
	ld hl,(displayPointer)
	inc hl
	inc hl
	add hl,de
	ex de,hl
	ld hl,(displayLastByte)
	or a
	sbc hl,de
	jp c,BCD_Display_EOL		;past the end of the number
	;Now we should check for screen clearance
	ld a,(curcol)
	cp 15
	jr nz,BCD_Display_Section_Loop
	ld a,(currow)
	push af
	ld a,05h
	B_CALL PutC
	pop af
	ld (currow),a
	or 1
	ret
BCD_Display_EOL:
	B_CALL EraseEOL
	xor a
	ret
DispA:
	;Displays the two digit number in A
	ld c,-1
DispA_TensDigit:
	inc c
	sub 10
	jr nc,DispA_TensDigit
	push af
	ld a,c
	or a
	jr z,DispA_OnesDigit
	add a,'0'
	B_CALL VPutMap
DispA_OnesDigit:
	pop af
	add a,'0'+10
	B_CALL VPutMap
	ret
LDispHL:
	;Displays the five digit value in HL
	ld ix,DispHL_Table
	ld b,4
LDispHL_Digit:
	ld c,-1
	ld e,(ix)
	ld d,(ix+1)
	inc ix
	inc ix
LDispHL_Digit_Loop:
	inc c
	or a
	sbc hl,de
	jr nc,LDispHL_Digit_Loop
	add hl,de
	;HL is the remainder
	ld a,c
	or a
	jr nz,LDispHL_ShowDigit
	bit 7,b
	jr z,LDispHL_NextDigit
LDispHL_ShowDigit:
	set 7,b
	ld a,c
	add a,'0'
	push bc
	push hl
	push ix
	B_CALL PutC
	pop ix
	pop hl
	pop bc
LDispHL_NextDigit:
	dec b
	ld a,b
	and 00001111b
	jr nz,LDispHL_Digit
	;Last digit
	ld a,l
	add a,'0'
	B_CALL PutC
	ret
DispHL_Table:
	DW 10000,1000,100,10
Scientific_Notation:
	;Input - HL is a number
	;Output - the number is displayed in scientific notation
	B_CALL EraseEOL
	ld ix,temp1
	res 0,(ix+0)			;set for rounding on last digit
	ld c,(hl)
	inc hl
	ld b,(hl)
	bit 7,b
	jr z,Scientific_Notation_Sign
	push hl
	push bc
	ld a,lneg
	B_CALL PutC
	pop bc
	pop hl
Scientific_Notation_Sign:
	res 7,b
	;For an integer of x digits, the power is x-1
	sla c
	rl b	;BC = BC * 2
	;BC is the number of digits
	push bc	;save it for later
	;Now display the first digit, then a decimal point, then five more
	inc hl
	ld a,(hl)
	and 11110000b
	ld a,(hl)
	jr nz,Scientific_Notation_DigitOne
	add a,a
	add a,a
	add a,a
	add a,a
	dec bc
	inc hl
	pop de
	push bc
Scientific_Notation_DigitOne:
	srl a
	srl a
	srl a
	srl a
	add a,'0'
	push bc
	push hl
	B_CALL PutC
	pop hl
	pop bc
	dec bc
	ld a,b
	or c
	jr z,Scientific_Notation_Done
	ld a,'.'
	push bc
	push hl
	B_CALL PutC
	pop hl
	pop bc
	;Now display five digits
	ld d,4
Scientific_Notation_DigitLoop:
	bit 0,c
	ld a,(hl)
	jr nz,Scientific_Notation_Odd
	and 11110000b
	srl a
	srl a
	srl a
	srl a
	dec hl
Scientific_Notation_Odd:
	and 00001111b
	bit 0,(ix+0)
	ret nz
	add a,'0'
	push bc
	push de
	push hl
	B_CALL PutC
	pop hl
	pop de
	pop bc
	inc hl
	dec d
	jr z,Scientific_Notation_LastDigitTime
	dec bc
	ld a,b
	or c
	jr nz,Scientific_Notation_DigitLoop
Scientific_Notation_Done:
	;Now display the exponent
	ld a,lexponent
	B_CALL PutC
	pop hl
	dec hl
	call LDispHL
	ret
Scientific_Notation_LastDigitTime:
	;We displayed the fourth digit...now to get the 5th we must round based on the 6th
	set 0,(ix+0)
	dec bc
	ld a,b
	or c
	jr z,Scientific_Notation_Done
	call Scientific_Notation_DigitLoop
	;A will contain the 5th digit
	ld d,a
	dec bc
	ld a,b
	or c
	ld a,d
	jr z,Scientific_Notation_Go
	inc hl
	push de
	call Scientific_Notation_DigitLoop
	add a,251
	pop bc
	ld a,b
Scientific_Notation_Go:
	adc a,'0'
	cp '0'+10
	jr c,Scientific_Notation_Go2
	dec a
Scientific_Notation_Go2:
	B_CALL PutC
	jr Scientific_Notation_Done
CountDigits:
	;Inputs - HL points to a number
	;Output - BC is the number of digits in that number, excluding any negative sign
	ld c,(hl)
	inc hl
	ld b,(hl)
	res 7,b
	inc hl
	ld a,(hl)
	sla c
	rl b
	and 11110000b
	ret nz
	dec bc
	ret
GetDigit:
	;Input - HL points to the number, BC is the digit to get (the first digit would be BC=1)
	;The negative sign does not count, and it is *assumed* that BC is a valid digit (not past end of number)
	;Output - Digit (0-9) returned in A
	inc hl
	inc hl
	ld a,(hl)
	dec bc
	and 11110000b
	jr nz,GetDigit_Regular
	inc bc
GetDigit_Regular:
	srl b
	rr c
	push af
	add hl,bc
	pop af
	ld a,(hl)
	jr c,GetDigit_Low
	srl a
	srl a
	srl a
	srl a
GetDigit_Low:
	and 00001111b
	ret
